import contextlib
with contextlib.redirect_stdout(None):
    import pygame

class Sound:
    def __init__(self, file: str, play_count: int=1, volume: float=1):
        pygame.init()
        pygame.mixer.set_num_channels(499)

        self.file = file
        self.volume = volume
        self.play_count = play_count
    
    def play(self):
        self.channel = pygame.mixer.find_channel()
        self.channel.set_volume(self.volume)
        self.channel.play(pygame.mixer.Sound(self.file), self.play_count)
    
    def stop(self):
        self.channel.pause()


from tkinter import *
from PIL import ImageTk, Image

class Rect:
    def __init__(self, parent, size: tuple=(10, 10), pos: tuple=(0, 0), color: str="black"):
        self.parent = parent.canvas
        self.size = size
        self.rect = self.parent.create_rectangle(pos[0] + size[0], pos[1] + size[1], pos[0], pos[1], fill=color)

    def move_right(self, speed: float):
        self.parent.move(self.rect, speed, 0)

    def move_left(self, speed: float):
        self.parent.move(self.rect, -speed, 0)

    def move_up(self, speed: float):
        self.parent.move(self.rect, 0, -speed)

    def move_down(self, speed: float):
        self.parent.move(self.rect, 0, speed)

class Oval:
    def __init__(self, parent, size: tuple=(10, 10), pos: tuple=(0, 0), color: str="black"):
        self.parent = parent.canvas
        self.size = size
        self.oval = self.parent.create_oval(pos[0] + size[0], pos[1] + size[1], pos[0], pos[1], fill=color)

    def move_right(self, speed: float):
        self.parent.move(self.oval, speed, 0)

    def move_left(self, speed: float):
        self.parent.move(self.oval, -speed, 0)

    def move_up(self, speed: float):
        self.parent.move(self.oval, 0, -speed)

    def move_down(self, speed: float):
        self.parent.move(self.oval, 0, speed)

class ImageRect(Rect):
    def __init__(self, parent, image_path: str, pos: tuple=(0, 0), size: tuple=(128, 128)):
        self.image = Image.open(image_path).resize(size)
        self.photo_image = ImageTk.PhotoImage(self.image)
        self.parent = parent.canvas
        self.parent_root = parent
        self.parent_root.root.one = self.photo_image
        self.rect = self.parent.create_image(pos, image=self.photo_image, anchor="nw")
    
    def rotate(self, angle: float, new_pos: tuple):
        self.image = self.image.rotate(angle)
        self.photo_image = ImageTk.PhotoImage(self.image)
        self.parent_root.root.one = self.photo_image
        self.rect = self.parent.create_image(new_pos, image=self.photo_image, anchor="nw")

class TextLabel:
    def __init__(self, parent, pos: tuple=(0, 0), **kwargs):
        self.parent = parent.canvas
        self.text = Label(self.parent, **kwargs)
        self.text.place(x=pos[0], y=pos[1])
    
    def change_text(self, text: str):
        self.text.config(text=text)

class InputText:
    def __init__(self, parent, pos: tuple=(0, 0), **kwargs):
        self.parent = parent.canvas
        self.input = Text(self.parent, **kwargs)
        self.input.place(x=pos[0], y=pos[1])

    def get(self):
        return self.input.get(1.0, "end-1c")
    
    def insert(self, text: str):
        self.input.insert(1.0, text)
    
    def clear(self):
        self.input.delete(1.0, "end")


import kriptik as kr
import time

win = kr.Window(
    size=(640, 640),
    color="white",
    fps=60
)

win.set_icon_bitmap(r"assets\AcidicV2.ico")

rect1 = kr.elements.Rect(
    win,
    size=(50, 50),
    pos=(10, 10),
    color="#F94D4D",
)

oval = kr.elements.Oval(
    win,
    size=(50, 50),
    pos=(110, 10),
    color="pink"
)

label = kr.elements.TextLabel(
    win,
    pos=(210, 10),
    text="Kriptik",
    width=7,
    height=1
)

input_ = kr.elements.InputText(
    win,
    pos=(310, 10),
    height=1,
    width=10
)

img = kr.elements.ImageRect(
    win,
    r"assets\king.png",
    pos=(410, 10),
    size=(64, 64)
)

def mainloop():
    global angle, img
    start_time = time.perf_counter()

    if kr.uinput.key_pressed("a"):
        img.move_left(10)
        rect1.move_left(10)
    if kr.uinput.key_pressed("d"):
        rect1.move_right(10)
        img.move_right(10)
    if kr.uinput.key_pressed("w"):
        rect1.move_up(10)
        img.move_up(10)
    if kr.uinput.key_pressed("s"):
        rect1.move_down(10)
        img.move_down(10)
    if kr.uinput.key_pressed("space"):
        kr.Sound(r"assets\sound.wav", play_count=1, volume=0.5).play()
    
    if kr.uinput.mouse_pressed("left_mouse"):
        img.rotate(45, win.element_pos(img.rect))

    label.change_text(f"FPS: {int(1/(time.perf_counter() - start_time))}")
    win.update()

loop = win.thread(mainloop)
loop.start()

win.start()


import threading
import time

class Thread:
    def __init__(self, func, fps: int=0):
        self.func = func
        self.fps = fps
        self.thread_on = False
    
    def thread(self):
        if self.fps == 0:
            while True:
                self.func()
        elif self.fps > 0:
            while True:
                self.func()
                time.sleep(1/self.fps)
        else:
            raise ValueError("\"fps\" argument must be above 0")
    
    def start(self):
        self.thread_on = True
        threading.Thread(target=self.thread).start()
    
    def stop(self):
        self.thread_on = False


import keyboard
import win32api

def key_pressed(val: str):
    if keyboard.is_pressed(val):
        return True
    else:
        return False
        
def mouse_pressed(val: str):
    if val == "left_mouse":
        val = 0x01
    elif val == "right_mouse":
        val = 0x02
    elif val == "middle_mouse":
        val = 0x04
    elif val == "side_mouse1":
        val = 0x05
    elif val == "side_mouse2":
        val = 0x06
    else:
        raise ValueError("\"val\" argument must be one of the following: left_mouse, right_mouse, middle_mouse, side_mouse1, side_mouse2")
    
    key_state = win32api.GetKeyState(val)

    if key_state < 0:
        return True
    else:
        return False


def sub_uint(x, y):
    result = x - y
    if result < 0:
        return -result

class Collision:
    def __init__(self, canvas, element1, element2):
        self.canvas = canvas
        self.e1 = self.canvas.bbox(element1)
        self.e2 = self.canvas.bbox(element2)
        self.a_y_len = sub_uint(self.e1[1], self.e1[3])
        self.b_x_len = sub_uint(self.e2[0], self.e2[2])
    
    def collision_loop(self, side_1, side_2):
        for x in range(self.e2[0], self.e2[0] + self.b_x_len):
            for i in range(self.e1[1], self.e1[1] + self.a_y_len):
                if self.e2[side_2] == i and self.e1[side_1] == x:
                    return True

    def collide(self):
        if self.collision_loop(2, 1): # right to top
            return True
        elif self.collision_loop(2, 3): # right to bottom
            return True
        if self.collision_loop(0, 1): # left to top
            return True
        elif self.collision_loop(0, 3): # left to bottom
            return True
        else:
            return False


from setuptools import setup, find_packages
import codecs
import os

VERSION = "0.0.2"
DESCRIPTION = "A 2D Python game engine"
LONG_DESCRIPTION = """
A 2D Python game engine with a reasonably wide range of functionality given it's size.
You can find it here on Github: https://github.com/wzack850/Kriptik
"""

# Setting up
setup(
    name="Kriptik",
    version=VERSION,
    author="wzack850",
    author_email="<wzack850@yahoo.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=["pillow", "pygame", "keyboard"],
    keywords=["python", "game engine", "2D", "python 3.x", "kriptik"],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


from tkinter import *
from PIL import ImageTk, Image

import uinput
import elements
from sound import Sound
from threads import Thread
from collision import Collision

def sub_uint(x, y):
    result = x - y
    if result < 0:
        return -result
    return result

class Window:
    def __init__(self, size: tuple=(500, 500), color: str="black", fps: int=0):
        self.root = Tk()
        self.title = "Kriptik"
        self.fullscreen = False
        self.mouse_pos = ()
        self.init()

        self.fps = fps
        self.size = size
        self.color = color
        self.canvas = Canvas(self.root, width=size[0], height=size[1], bg=color)
        self.canvas.pack()

    def init(self):
        self.root.title(self.title)
        self.root.attributes("-fullscreen", self.fullscreen)
        self.root.bind("<Motion>", self.mp)

    def set_icon_bitmap(self, image_path: str):
        self.root.wm_iconbitmap(image_path)

    def update(self):
        self.canvas.update()

    def start(self):
        self.root.mainloop()

    def element_pos(self, element):
        return tuple(self.canvas.coords(element))

    def delete_element(self, element):
        self.canvas.delete(element)
    
    def mp(self, event):
        self.mouse_pos = (event.x, event.y)

    def thread(self, func):
        return Thread(func, self.fps)
        
    def collide(self, element1, element2):
        return Collision(self.canvas, element1, element2).collide()